Option Strict On
Option Compare Text

Public Class MySldWorksCls

    Dim oSwAppCls As SldWorks.SldWorks

    Friend ReadOnly Property MySldWorks() _
                    As SldWorks.SldWorks
        Get
            Return oSwAppCls
        End Get
    End Property

    Friend Function SolidWorksInstanzCreate() _
                        As SldWorks.SldWorks
        Try
            'SolidWorks Objekt initialisieren
            oSwAppCls = CType(Microsoft.VisualBasic.CreateObject( _
                  ProgId:="SldWorks.Application"), SldWorks.SldWorks)
            Return oSwAppCls
        Catch ex As Exception
            Debug.Assert(False)
            MsgBox("Fehler: Wo: " & _
                ex.StackTrace & " Was: " & ex.Message)
            Return Nothing
        End Try
    End Function

    Friend Sub VersionAnzeigen()
        Try
            'Rckgabewert der SolidWorks 
            'MessageBox
            Dim eSwMsgBox As SwConst.swMessageBoxResult_e
            'SolidWorks Fenster anzeigen
            oSwAppCls.Visible = True
            'SolidWorks Version
            'in einer MsgBox anzeigen
            eSwMsgBox = CType(oSwAppCls.SendMsgToUser2( _
                "SolidWorks Version: " & _
                Me.VersionInJahr, _
                SwConst.swMessageBoxIcon_e.swMbInformation, _
                SwConst.swMessageBoxBtn_e.swMbYesNo), _
                SwConst.swMessageBoxResult_e)
            Select Case eSwMsgBox
                Case SwConst.swMessageBoxResult_e.swMbHitYes
                    'Hinweis in einer MsgBox
                    MsgBox("Der User hat auf Ja geklickt")
                Case SwConst.swMessageBoxResult_e.swMbHitNo
                    'Hinweis in einer MsgBox
                    MsgBox("Der User hat auf Nein geklickt")
                Case Else
                    Debug.Assert(False)
            End Select

        Catch ex As Exception
            Debug.Assert(False)
            MsgBox("Fehler: Wo: " & _
                ex.StackTrace & " Was: " & ex.Message)
        End Try
    End Sub

    Friend Function SolidWorksInstanzGet() _
                      As SldWorks.SldWorks
        Try
            oSwAppCls = CType(Microsoft.VisualBasic.GetObject( _
                Class:="SldWorks.Application"), SldWorks.SldWorks)
            Return oSwAppCls
        Catch ex As Exception
            Debug.Assert(False)
            MsgBox("Fehler: Wo: " & _
                ex.StackTrace & " Was: " & ex.Message)
            Return Nothing
        End Try
    End Function

    ''' <summary>
    ''' Initialisiert das SolidWorks Objekt durch die Methode GetObject 
    ''' und fragt in einer MsgBox nach wenn das Objekt nicht greifbar ist. 
    ''' </summary>
    ''' <param name="bUserAbbruch">
    ''' Optionaler Rckgabewert, welcher mitteilt ob der User
    ''' die Aktion abgerochen hat oder ein Fehler aufgetreten ist.
    ''' </param>
    ''' <returns>Instanz der atkuellen SolidWorks Sitzung</returns>
    ''' <remarks></remarks>
    Public Function SolidWorksInstanz( _
            Optional ByRef bUserAbbruch As Boolean = False) _
            As SldWorks.SldWorks
        Try
            'Variable fr die MsgBox-Rckgabe
            Dim lDialogStatus As Microsoft.VisualBasic.MsgBoxResult
            Try
                'SolidWorks Objekt belegen
                oSwAppCls = CType(GetObject(, _
                    "SldWorks.Application"), _
                    SldWorks.SldWorks)
            Catch ex As Exception
            End Try
            'Wenn das SolidWorks Objekt nicht belegt ist...
            If oSwAppCls Is Nothing Then
                '... in einer MsgBox nachfragen
                lDialogStatus = MsgBox( _
                    "Fehler beim Aufbau einer Schnittstelle zu SolidWorks." & _
                    vbNewLine & _
                    "Bitte ffnen Sie SolidWorks und klicken Sie auf ""Ok""." & _
                    vbNewLine & _
                    "Sollte SolidWorks bereits geffnet sein, " & _
                    "schlieen Sie SolidWorks, " & _
                    vbNewLine & _
                    "ffnen Sie es erneut und klicken Sie dann auf ""Ok""." & _
                    vbNewLine & _
                    "Wenn Sie denn Startvorgang abbrechen " & _
                    "mchten klicken Sie auf ""Abbrechen"".", _
                    MsgBoxStyle.Information Or MsgBoxStyle.OkCancel, _
                    "Schuler Design Automation GmbH")
                'Rckgabewert der MsgBox auswerten
                Select Case lDialogStatus
                    Case MsgBoxResult.Ok
                        'User hat SolidWorks geffnet
                        'Funktion einfach nochmal ausfhren
                        Return SolidWorksInstanz(bUserAbbruch)
                    Case MsgBoxResult.Cancel
                        'User mchte die Anwednung abbrechen
                        bUserAbbruch = True
                        Return Nothing
                End Select
            End If
            Return oSwAppCls
        Catch ex As Exception
            Debug.Assert(False)
            MsgBox("Fehler: Wo: " & _
                ex.StackTrace & " Was: " & ex.Message)
            Return Nothing
        End Try
    End Function

    Friend ReadOnly Property VersionInJahr() As Integer
        Get
            Dim iHauptversion As Integer
            Dim sVersionen() As String
            'SolidWorks Instanz prfen
            If oSwAppCls Is Nothing Then
                '-1 als ungltige Rckgabe verwenden
                Return -1
            Else
                'Version ermitteln und an den
                'Trennpunkten teilen
                sVersionen = _
                    oSwAppCls.RevisionNumber.Split( _
                    CChar("."))
                'Nur die Hauptversion weiter verwenden
                iHauptversion = CInt(sVersionen(0))
                'Version in Jahreszahl umrechnen
                iHauptversion = iHauptversion + 1992
                'Die Hauptversion in Jahren 
                'als Rckgabe verwenden
                Return iHauptversion
            End If
        End Get
    End Property

    ''' <summary>
    ''' ffnet das bergebene SolidWorks Dokument in 
    ''' SolidWorks und gibt eine Instanz auf das 
    ''' SldWorks.ModelDoc2 Objekt zurck
    ''' </summary>
    ''' <param name="sDateipfad">
    ''' Dateipfad des SolidWorks Dokuments, 
    ''' welches geffnet werden soll</param>
    ''' <param name="eOpenDocOptions">
    ''' Optionaler bergabeparameter, 
    ''' welcher die ffnenoption fr SolidWorks enthlt</param>
    ''' <param name="eOpenWarning">
    ''' Optionaler Rckgabeparameter, 
    ''' welchen den Warningstatus von SolidWorks enthlt</param>
    ''' <returns>Instanz eines ModelDoc2 Objekts 
    ''' des geffneten Dokuments. Bei einem Fehler Nothing</returns>
    ''' <remarks></remarks>
    Friend Function DokumentOeffnen( _
            ByVal sDateipfad As String, _
            Optional ByVal eOpenDocOptions As SwConst.swOpenDocOptions_e = SwConst.swOpenDocOptions_e.swOpenDocOptions_Silent, _
            Optional ByRef eOpenWarning As SwConst.swFileLoadWarning_e = 0) As SldWorks.ModelDoc2
        Try
            'Bentigte Variablen
            Dim oSwModel As SldWorks.ModelDoc2
            Dim eError As SwConst.swFileLoadError_e
            'SolidWorks Dokumenttyp
            Dim eDocTyp As SwConst.swDocumentTypes_e
            Dim sExtension As String
            'Existenz der bergeben Datei prfen
            If IO.File.Exists(sDateipfad) = False Then
                'Fehler erzeugen
                Throw New Exception( _
                    "Die Datei: " & sDateipfad & _
                    " ist nicht vorhanden.")
            End If
            'SolidWorks Application Objekt prfen
            If oSwAppCls Is Nothing Then
                'Fehler erzeugen
                Throw New Exception( _
                    "Kein SolidWorks Application " & _
                    "Objekt vorhanden")
            End If
            'Dokumenttyp ermitteln
            sExtension = IO.Path.GetExtension( _
                                    sDateipfad)
            Select Case sExtension
                Case ".sldprt"
                    eDocTyp = _
                    SwConst.swDocumentTypes_e.swDocPART
                Case ".sldasm"
                    eDocTyp = _
                    SwConst.swDocumentTypes_e.swDocASSEMBLY
                Case ".slddrw"
                    eDocTyp = _
                    SwConst.swDocumentTypes_e.swDocDRAWING
                Case Else
                    'Fehler erzeugen
                    Throw New Exception( _
                    "Ungltiger Dokumenttyp")
            End Select
            'berprfungen erfolgreich
            'Dokument in SolidWorks ffnen
            oSwModel = oSwAppCls.OpenDoc6( _
                sDateipfad, _
                eDocTyp, eOpenDocOptions, _
                "", _
                CType(eError, SwConst.swFileLoadError_e), _
                CType(eOpenWarning, SwConst.swFileLoadWarning_e))
            'Rckgabewerte auswerten
            If oSwModel Is Nothing Then
                'Fehler erzeugen
                Throw New Exception( _
                "Das Dokument: " & _
                sDateipfad & _
                " konnte nicht in SolidWorks geffnet werden. " & _
                "Fehlerantwort: " & eError.ToString)
            End If
            'Es hat alles geklappt
            Return oSwModel
        Catch ex As Exception
            Debug.Assert(False)
            MsgBox("Fehler: Wo: " & _
                ex.StackTrace & " Was: " & ex.Message)
            Throw New Exception( _
                "Fehler beim ffnen des Dokuments: " & _
                sDateipfad & ".")
            Return Nothing
        End Try
    End Function

    Friend ReadOnly Property AktuellesDokument() As SldWorks.ModelDoc2
        Get
            'SolidWorks Instanz prfen
            If oSwAppCls Is Nothing Then
                'Nothing als ungltige Rckgabe verwenden
                Return Nothing
            Else
                'Aktuelles Dokument zurckgeben
                Return CType( _
                    oSwAppCls.ActiveDoc, _
                    SldWorks.ModelDoc2)
            End If
        End Get
    End Property

    Friend Sub DokumentSchliessen()
        Try
            'SolidWorks Instanz prfen
            If oSwAppCls Is Nothing Then
                'Fehler
                Throw New Exception( _
                "Keine SolidWorks Instanz vorhanden.")
            Else
                'Aktuell in SolidWorks 
                'aktives Dokument speichern
                oSwAppCls.CloseDoc( _
                    Me.AktuellesDokument.GetTitle)
                'Fr das Testen der verschiedenen
                'SolidWorks API Methoden_
                'UserControl Einstellung festlegen
                'oSwAppCls.UserControl = True
                'Dokument mit ModelDoc2.Close schlieen
                'Me.AktuellesDokument.Close()
                'Dokument mit ModelDoc2.Quit schlieen
                'Me.AktuellesDokument.Quit()
                'Dokument mit SldWorks.CloseDoc schlieen
                'oSwAppCls.CloseDoc(Me.AktuellesDokument.GetTitle)
                'Dokument mit SldWorks.QuitDoc schlieen
                'oSwAppCls.QuitDoc(Me.AktuellesDokument.GetTitle)
            End If
        Catch ex As Exception
            Debug.Assert(False)
            MsgBox("Fehler: Wo: " & _
                ex.StackTrace & " Was: " & ex.Message)
            Throw New Exception( _
            "Fehler beim Schliessen " & _
            "des aktuellen Dokuments.")
        End Try
    End Sub

    Friend Sub DokumentSpeichern( _
            Optional ByVal eSaveAsOptions As SwConst.swSaveAsOptions_e = _
                SwConst.swSaveAsOptions_e.swSaveAsOptions_Silent, _
            Optional ByRef eSaveWarning As SwConst.swFileSaveWarning_e = 0)
        Try
            'SolidWorks Instanz prfen
            If oSwAppCls Is Nothing Then
                'Fehler
                Throw New Exception( _
                "Keine SolidWorks Instanz vorhanden.")
            Else
                'Rckgabewerte
                Dim bStatus As Boolean
                Dim eError As SwConst.swFileSaveError_e
                'Aktuelles Dokument speichern
                bStatus = Me.AktuellesDokument.Save3( _
                       eSaveAsOptions, _
                       CType(eError, SwConst.swFileSaveError_e), _
                       CType(eSaveWarning, SwConst.swFileSaveWarning_e))
                'Rckgaben auswerten
                If bStatus = False Then
                    Throw New Exception( _
                    "Aktuelles Dokument wurde nicht gespeichert." & _
                    vbNewLine & "Errormeldung der API Methode: " & _
                    eError.ToString & ".")
                End If
            End If
        Catch ex As Exception
            Debug.Assert(False)
            MsgBox("Fehler: Wo: " & _
                ex.StackTrace & " Was: " & ex.Message)
            Throw New Exception( _
            "Fehler beim Speichern " & _
            "des aktuellen Dokuments.")
        End Try
    End Sub

    Friend Sub DokumentSpeichernUnter( _
    ByVal sSpeicherpfad As String, _
    Optional ByVal eSaveAsOptions As SwConst.swSaveAsOptions_e = _
    SwConst.swSaveAsOptions_e.swSaveAsOptions_Silent, _
    Optional ByRef eSaveWarning As SwConst.swFileSaveWarning_e = 0)
        Try
            'SolidWorks Instanz prfen
            If oSwAppCls Is Nothing Then
                'Fehler
                Throw New Exception( _
                "Keine SolidWorks Instanz vorhanden.")
            Else
                'Rckgabewerte
                Dim bStatus As Boolean
                Dim eError As SwConst.swFileSaveError_e
                'Aktuelles Dokument speichern
                bStatus = Me.AktuellesDokument.SaveAs4( _
                       sSpeicherpfad, _
                       SwConst.swSaveAsVersion_e.swSaveAsCurrentVersion, _
                       eSaveAsOptions, _
                       CType(eError, SwConst.swFileSaveError_e), _
                       CType(eSaveWarning, SwConst.swFileSaveWarning_e))
                'Rckgaben auswerten
                If bStatus = False Then
                    Throw New Exception( _
                    "Aktuelles Dokument wurde nicht gespeichert." & _
                    vbNewLine & "Errormeldung der API Methode: " & _
                    eError.ToString & ".")
                End If
            End If
        Catch ex As Exception
            Debug.Assert(False)
            MsgBox("Fehler: Wo: " & _
                ex.StackTrace & " Was: " & ex.Message)
            Throw New Exception( _
            "Fehler beim ""Speichern unter"" " & _
            "des aktuellen Dokuments.")
        End Try
    End Sub

    Friend ReadOnly Property AktuellesDokumentSpeicherpfad() As String
        Get
            'Aktuelles Dokument prfen
            If Me.AktuellesDokument Is Nothing Then
                'Leerer String als ungltige Rckgabe verwenden
                Return ""
            Else
                'Speicherpfad zurckgeben
                Return Me.AktuellesDokument.GetPathName
            End If
        End Get
    End Property

    Friend ReadOnly Property AktuellesDokumentExtension() As String
        Get
            'Speicherpfad aufschlsseln
            Return IO.Path.GetExtension(Me.AktuellesDokumentSpeicherpfad)
        End Get
    End Property

    Friend ReadOnly Property AktuellesDokumentVerzeichnis() As String
        Get
            'Speicherpfad aufschlsseln
            Return IO.Path.GetDirectoryName(Me.AktuellesDokumentSpeicherpfad)
        End Get
    End Property

    Friend ReadOnly Property AktuellesDokumentDateiName() As String
        Get
            'Speicherpfad aufschlsseln
            Return IO.Path.GetFileName(Me.AktuellesDokumentSpeicherpfad)
        End Get
    End Property

    ''' <summary>
    ''' Ermittelt die Referenzen eines SolidWorks Dokument 
    ''' und gibt diese als String Liste zurck.
    ''' Das Dokument kann durch den Optionalen Parameter 
    ''' sDokumentPfad angegeben werden. Ist dies nicht der Fall 
    ''' werden die Referenzen des aktuell in 
    ''' SolidWorks geffneten Dokuments ermittelt.
    ''' </summary>
    ''' <param name="sDokumentPfad">Optionaler Parameter. 
    ''' Wird ein Dokumentpfad bergeben, 
    ''' werden die Referenzen dieses Dokuments ermitteln</param>
    ''' <param name="bAuchReadOnlyInfo">Optionaler Parameter. 
    ''' Bestimmt ob die ReadOnly Informationen ermittelt werden.
    ''' Diese sind jedoch nie in der Stringliste enthalten</param>
    ''' <returns>Referenzen des ausgelesenen Dokuments. 
    ''' Wenn dieses keine Referenzen besitz Nothing</returns>
    ''' <remarks></remarks>
    Friend Function ReferenzenEinesDokumentsErmitteln( _
        Optional ByVal sDokumentPfad As String = "", _
        Optional ByVal bAuchReadOnlyInfo As Boolean = False) As _
        System.Collections.Generic.List(Of String)
        Try
            Dim iStep As Integer
            'Dummy SaveArray
            Dim oDummyArray() As Object
            'Referenzliste
            Dim scReferenzen As _
                System.Collections.Generic.List(Of String)
            'SolidWorks Application Objekt prfen
            If oSwAppCls Is Nothing Then
                'Fehler erzeugen
                Throw New Exception( _
                "Keine SolidWorks Instanz vorhanden.")
            End If
            'Existenz der bergeben Datei prfen
            If IO.File.Exists(sDokumentPfad) = False Then
                'ber das ModelDoc2 ermitteln
                oDummyArray = CType( _
                    Me.AktuellesDokument.GetDependencies2( _
                    True, False, False), [Object]())
            Else
                'ber die SolidWorks Application
                oDummyArray = CType( _
                    oSwAppCls.GetDocumentDependencies2( _
                    sDokumentPfad, True, _
                    False, False), [Object]())
            End If
            'berprfen ob das Dokument
            'berhaupt Referenzen hat!
            If IsArray(oDummyArray) = False Then
                'Wenn keine Referenzen gefunden wurden, 
                'war das kein Fehler der Funktion!
                Return Nothing
            End If
            'Parameter addReadOnlyInfo bercksichtigen
            If bAuchReadOnlyInfo Then
                iStep = 3
            Else
                iStep = 2
            End If
            'Stringliste initialisieren
            scReferenzen = New _
                System.Collections.Generic.List(Of String)
            'SaveArray aufschlsseln
            'und Dokumentpfade verarbeiten
            For i As Integer = 1 To _
                oDummyArray.GetUpperBound(0) Step iStep
                scReferenzen.Add(CStr(oDummyArray(i)))
            Next
            Return scReferenzen
        Catch ex As Exception
            Debug.Assert(False)
            MsgBox("Fehler: Wo: " & _
             ex.StackTrace & " Was: " & ex.Message)
            Return Nothing
        End Try
    End Function

    Friend Function DokumentEigenschaftenUndWertErmitteln( _
            Optional ByVal sKonfiguration As String = "") As _
            System.Collections.Generic.Dictionary(Of String, String)
        Try
            'Dictionary fr die ermittelten
            'Dokumenteigenschaften und dessen Wert
            Dim scEigenschaftenUndWert As _
                System.Collections.Generic.Dictionary( _
                Of String, String)
            'Je nach SolidWorks Version die richtige Methode ausfhren
            If Me.VersionInJahr >= 2007 Then
                'Ab SolidWorks 2007
                scEigenschaftenUndWert = _
                    Me.DokumentEigenschaftenUndWertErmittelnAbSw2007( _
                    sKonfiguration)
            Else
                'Fr ltere SolidWorks Versionen
                scEigenschaftenUndWert = _
                    Me.DokumentEigenschaftenUndWertErmittelnVorSw2007( _
                    sKonfiguration)
            End If

            Return scEigenschaftenUndWert
        Catch ex As Exception
            Debug.Assert(False)
            MsgBox("Fehler: Wo: " & _
             ex.StackTrace & " Was: " & ex.Message)
            Return Nothing
        End Try
    End Function

    Private Function DokumentEigenschaftenUndWertErmittelnVorSw2007( _
            Optional ByVal sKonfiguration As String = "") As _
            System.Collections.Generic.Dictionary(Of String, String)
        Try
            'Dictionary fr die ermittelten
            'Dokumenteigenschaften und dessen Wert
            Dim scEigenschaftenUndWert As _
                System.Collections.Generic.Dictionary( _
                Of String, String)
            'Stringarray fr die vorhandenen
            'Dokumenteigenschaften des Dokuments
            Dim sEigenschaften() As String
            'SolidWorks Application Objekt prfen
            If oSwAppCls Is Nothing Then
                'Fehler erzeugen
                Throw New Exception( _
                "Keine SolidWorks Instanz vorhanden.")
            End If
            'Dokumenteigenschaften des aktuellen
            'Dokuments ermitteln
            sEigenschaften = CType( _
                Me.AktuellesDokument.GetCustomInfoNames2( _
                sKonfiguration), String())
            'Dictionary initialisieren
            scEigenschaftenUndWert = New _
                System.Collections.Generic.Dictionary( _
                Of String, String)
            'Alle Eigenschaften bercksichtigen
            'und dessen Wert ermitteln
            For iEigenschaft As Integer = _
                        0 To sEigenschaften.GetUpperBound(0)
                'Eigenschaften und dessen ermitteln
                'Wert als Eintrag in die Dictionary
                scEigenschaftenUndWert.Add( _
                    sEigenschaften(iEigenschaft), _
                    Me.AktuellesDokument.GetCustomInfoValue( _
                    sKonfiguration, sEigenschaften(iEigenschaft)))
            Next
            Return scEigenschaftenUndWert
        Catch ex As Exception
            Debug.Assert(False)
            MsgBox("Fehler: Wo: " & _
             ex.StackTrace & " Was: " & ex.Message)
            Return Nothing
        End Try
    End Function

    Private Function DokumentEigenschaftenUndWertErmittelnAbSw2007( _
      Optional ByVal sKonfiguration As String = "") As _
      System.Collections.Generic.Dictionary(Of String, String)
        Try
            'Dictionary fr die ermittelten
            'Dokumenteigenschaften und dessen Wert
            Dim scEigenschaftenUndWert As _
                System.Collections.Generic.Dictionary( _
                Of String, String)
            'Stringarray fr die vorhandenen
            'Dokumenteigenschaften des Dokuments
            Dim sEigenschaften() As String
            'Inhalt der Dokumenteigenschaft
            Dim sInhalt As String = Nothing
            'Erweiterungsobjekt eines 
            'SolidWorks Dokuments
            Dim oModelDocExt As SldWorks.ModelDocExtension
            'Objekt fr die Eigenschaften 
            'eines Dokuments
            Dim oCustomPropertyManager As _
                SldWorks.CustomPropertyManager
            'SolidWorks Application Objekt prfen
            If oSwAppCls Is Nothing Then
                'Fehler erzeugen
                Throw New Exception( _
                "Keine SolidWorks Instanz vorhanden.")
            End If
            'Erweiterungsobjekt des 
            'Dokuments initialisieren
            oModelDocExt = Me.AktuellesDokument.Extension
            'Dokumenteigenschaftenmanger
            'des aktuell Dokuments initialisieren
            oCustomPropertyManager = _
                oModelDocExt.CustomPropertyManager( _
                sKonfiguration)
            sEigenschaften = CType( _
                oCustomPropertyManager.GetNames, String())
            'Dictionary initialisieren
            scEigenschaftenUndWert = New _
                System.Collections.Generic.Dictionary( _
                Of String, String)
            'Alle Eigenschaften bercksichtigen
            'und dessen Wert ermitteln
            For iEigenschaft As Integer = _
                        0 To sEigenschaften.GetUpperBound(0)
                'Inhalt der Eigenschaft ermitteln
                oCustomPropertyManager.Get2( _
                    sEigenschaften(iEigenschaft), sInhalt, Nothing)
                'Eigenschaften und dessen ermitteln
                'Wert als Eintrag in die Dictionary
                scEigenschaftenUndWert.Add( _
                    sEigenschaften(iEigenschaft), _
                    sInhalt)
            Next
            Return scEigenschaftenUndWert
        Catch ex As Exception
            Debug.Assert(False)
            MsgBox("Fehler: Wo: " & _
             ex.StackTrace & " Was: " & ex.Message)
            Return Nothing
        End Try
    End Function

End Class

